@@ -52,6 +52,9 @@ struct rdpup_os_bitmap
 enum shared_memory_status {
     SHM_UNINITIALIZED = 0,
     SHM_RESIZING,
+    SHM_ACTIVE_PENDING,
+    SHM_RFX_ACTIVE_PENDING,
+    SHM_H264_ACTIVE_PENDING,
     SHM_ACTIVE,
     SHM_RFX_ACTIVE,
     SHM_H264_ACTIVE
@@ -68,8 +71,6 @@ struct _rdpClientCon
     struct stream *out_s;
     struct stream *in_s;
 
-    int rectIdAck;
-    int rectId;
     int connected; /* boolean. Set to False when I/O fails */
     int begin; /* boolean */
     int count;
@@ -106,7 +107,7 @@ struct _rdpClientCon
     struct xrdp_client_info client_info;
 
     uint8_t *shmemptr;
-    int shmemid;
+    int shmemfd;
     int shmem_bytes;
     int shmem_lineBytes;
     RegionPtr shmRegion;
@@ -121,8 +122,8 @@ struct _rdpClientCon
 
     RegionPtr dirtyRegion;
 
-    int num_rfx_crcs_alloc;
-    int *rfx_crcs;
+    int num_rfx_crcs_alloc[16];
+    int *rfx_crcs[16];
 
     /* true = skip drawing */
     int suppress_output;
@@ -137,9 +138,6 @@ extern _X_EXPORT int
 rdpClientConEndUpdate(rdpPtr dev, rdpClientCon *clientCon);
 extern _X_EXPORT int
 rdpClientConSetFgcolor(rdpPtr dev, rdpClientCon *clientCon, int fgcolor);
-extern _X_EXPORT void
-rdpClientConSendArea(rdpPtr dev, rdpClientCon *clientCon,
-                     struct image_data *id, int x, int y, int w, int h);
 extern _X_EXPORT int
 rdpClientConFillRect(rdpPtr dev, rdpClientCon *clientCon,
                      short x, short y, int cx, int cy);
@@ -183,5 +181,10 @@ extern _X_EXPORT int
 rdpClientConSetCursorEx(rdpPtr dev, rdpClientCon *clientCon,
                         short x, short y, uint8_t *cur_data,
                         uint8_t *cur_mask, int bpp);
+extern _X_EXPORT int
+rdpClientConSetCursorShmFd(rdpPtr dev, rdpClientCon *clientCon,
+                           short x, short y,
+                           uint8_t *cur_data, uint8_t *cur_mask, int bpp,
+                           int width, int height);
 
 #endif
